/*
 * FILE --- tank_sim.cc
 *
 * PURPOSE --- To provide an example use of the class for 4'th
 * order Runge-Kutta simulation, RK4_SIM, and the class for implementing
 * n-input m-output adaptive Radial Basis Function (RBF) neural network (NN).
 *
 * Implements the member functions of a class for performing a simulation of
 * a simple surge tank level dynamics controlled by a RBF adaptive controller.
 *
 */

// Include Files
#include "tank_sim.hh"

TANK_SIM::TANK_SIM (double _sampling_period, int _simulation_per_cycle,
		    double _initial_time, double *_initial_state) 
  : RK4_SIM(1, 1, (_sampling_period/((double)_simulation_per_cycle)), 
	    _initial_time, _initial_state)
{
  // samp_period >= integration_step
  
  // Initialize parameters
  sampling_period = _sampling_period;
  simulation_per_cycle = _simulation_per_cycle;
  
  a = 1.0;
  b = 2.0;
  c = 1.0;
  g = 9.8;
}

TANK_SIM::~TANK_SIM()
{
  
}

inline void TANK_SIM::DERIV(const double t, const double *x, 
			    const double *u, double *xdot)
{
  // This is where the derivative is defined: xdot = f(t,x,u)
  double x0 = 0.001;

  // Make sure that the level in the tank is positive 
  // (because of the sqrt in the dynamics)
  if (x[0] < 0.0)
    {
      xdot[0] = (-c*sqrt(2.0*g*x0) + u[0])/(a*x0*x0 + b);
    }
  else
    {
      // Surge Tank Dynamics
      xdot[0] = (-c*sqrt(2.0*g*x[0]) + u[0])/(a*x[0]*x[0] + b);
    }
}

// Each sampling time based on the current control input 
// the tank ship dynamics are simulated
void TANK_SIM::simulate(double _input)
{
  // Assign the control input 
  ctrl_input[0] = _input;

  // Make sure that the level in the tank is positive 
  // (because of the sqrt in the dynamics)
  if (plant_state[0] < 0.0)
    {
      plant_state[0] = 0.001;
    }
  
   // Simulate one sampling period
  for (int ii=0; ii < simulation_per_cycle; ii++)
    {
      solve();
    }
}


